import ast
from typing import Type, Optional, Any

from core.typing import T


def get_value(
    source: dict,
    key: str,
    cast_type: Type[T],
    default: Optional[T] = None,
    list_delimiter: str = ",",
) -> Optional[T]:
    """
    Get value from source dictionary and cast it to a specified type.
    :param dict source: The source dictionary.
    :param str key: The name of the environment variable.
    :param Type[T] cast_type: A callable to cast the variable's value.
    :param Optional[T] default: The default value to return if the variable is not found. Defaults to None.
    :param str list_delimiter: The delimiter to split the value into a list. Defaults to ",".
    :return: The casted value, or None if the key is not found.
    :rtype: Optional[T]
    """

    if key in source:
        value = source[key]
        if isinstance(value, cast_type):
            return value

        try:
            if cast_type == bool:
                return value.lower() in ["true", "1"]

            if cast_type == list:
                subtype = (
                    cast_type.__args__[0] if hasattr(cast_type, "__args__") else None
                )
                value = ast.literal_eval(value)
                return [
                    subtype(item) if subtype is not None else item for item in value
                ]

            return cast_type(value)
        except (ValueError, TypeError):
            return default
    else:
        return default
